CCA <- function(N, Z, inertia=0) {
 
# function CCA for performing CCA on response matrix N with explanatory variables in matrix Z
# (note that no checking is done on the matrix N, which should be nonnegative and have nonzero row and column sums)

# outputs: -- principal row coordinates ARPC, contribution column coordinates ACCC
#               and singular values ASV of constrained solution
#          -- principal row coordinates BRPC, contribution column coordinates BCCC
#               and singular values BSV of residual unconstrained solution
#          -- total inertia TV, constrained inertia AV, unconstrained inertia BV 

# if inertia = 1 then only constrained inertia AV is output (for permutation test purposes)

  n <- sum(N)
  P <- N/n
  r <- apply(P,1,sum)
  c <- apply(P,2,sum)
  Dr <- diag(r)
  Dc <- diag(c)

# prepare "predictor" data matrix

  Corr <- cor(Z)
  OZ <- Z
  one <- rep(1,nrow(Z))
  Z <- Z-one%*%t(r)%*%Z
  S <- t(Z)%*%Dr%*%Z
  wvar <- diag(S)
  Z <- Z%*%sqrt(diag(1/wvar))

# weighted correlation matrix

  S <- t(Z)%*%Dr%*%Z

# inverse of weighted correlation matrix

  S.svd <- svd(S)
  k <- length(which(S.svd$d > 1.e-10))
  invS <- S.svd$u[,1:k] %*% diag(1/S.svd$d[1:k]) %*% t(S.svd$v[,1:k])

# projection matrix

  Proj <- sqrt(Dr)%*%Z%*%invS%*%t(Z)%*%sqrt(Dr)

# standardized residuals

  Resid <- diag(sqrt(1/r))%*%(P-r%*%t(c))%*%diag(sqrt(1/c))

# matrix A for constrained SVD

  A <- Proj%*%Resid

# constrained SVD & singular values & coordinates

  svdA <- svd(A)
  ASV <- svdA$d
  if(inertia==0) {

    ARPC <- diag(sqrt(1/r))%*% svdA$u %*% diag(svdA$d)
    ACCC <- svdA$v

# matrix B for unconstrained SVD

    B <- (diag(rep(1,nrow(Z)))-Proj)%*%Resid

# unconstrained SVD & singular values & coordinates

    svdB<-svd(B)
    BSV <- svdB$d
    BRPC <- diag(sqrt(1/r))%*% svdB$u
    BCCC <- svdB$v
  }

  TV <- sum(Resid^2)
  AV <- sum(ASV^2)
  if(inertia==0) BV <- sum(BSV^2)
  if(inertia==0) CCA.output <- list(ARPC=ARPC,ACCC=ACCC,AV=AV,BRPC=BRPC,BCCC=BCCC,BV=BV,TV=TV)
  if(inertia==1) CCA.output <- list(AV=AV)
  return(CCA.output)
}
    