fuzzy.tri <- function(x, H=0, hinge)

# Function to convert a vector x of values (a single variable) into H fuzzy coded categories
#  using triangular membership functions
# By default the hinges used are the quantiles of the variable's distribution
# For user-defined hinges, a vector 'hinges' is provided

# Examples of use, where x is a vector of length n:

#    fuzzy.tri(x,3) yields a nx3 matrix with the fuzzy categories as columns; the hinge points 
#                   used are the minimum, median and maximum value
#    fuzzy.tri(x, hinges=c(0,10,50,100))  yields a nx4 matrix with the fuzzy categories as columns;
#                   the hinge points are 0, 10, 50 and 100.   

 {
  
   # Triangular membership coding of values into H categories. 

   if (!is.vector(x)) stop("Only vectors handled currently as input.\n")
   if (H==1) stop("Number of categories (H) must be at least 2.\n")


   # Define H hinges if H=2 or more

   if(H>=2) {

     hinge    <- rep(0,H)
     hinge[1] <- min(x)
     hinge[H] <- max(x)
     if(H>2) {
       for(h in 2:H) hinge[h]  <- quantile(x, (h-1)/(H-1))
     }
   }

   if(H==0) H <- length(hinge)
   if(H==0) stop("If H=0 then hinge points have to be provided.\n")

   plin.x <- matrix(0, nrow=length(x), ncol=H)

   # first
   
   plin.x[x <= hinge[2], 1]  <- (hinge[2] - x[x <= hinge[2]]) / (hinge[2] - hinge[1])

   # last
   
   plin.x[x > hinge[H-1], H] <- (x[x > hinge[H-1]] - hinge[H-1]) / (hinge[H] - hinge[H-1])


   # possible categories in-between

   if(H>2) {

     for(h in 2:(H-1)) {

       plin.x[(x <= hinge[h]) & (x > hinge[h-1]), h] <- (x[(x <= hinge[h]) & (x > hinge[h-1])] - hinge[h-1]) / (hinge[h] - hinge[h-1])
       plin.x[(x > hinge[h]) & (x <= hinge[h+1]), h] <- (hinge[h+1] - x[(x > hinge[h]) & (x <= hinge[h+1])]) / (hinge[h+1] - hinge[h])
   
     }
   }

   plin.x

 }

