# ---------------------------------------------------------------------------------------------------
# script to compute adjusted R^2 values for CCA (Peres-Neto et al., Ecology, 2006) using function CCA
# and to illustrate fuzzy coding using function fuzzy.tri

# read artificial data and set up biological abundance data in bio, explanatory variables in X and Y

artificial <- read.csv("artificial.csv")
bio <- as.matrix(artificial[,1:5])
X <- artificial[,6]
Y <- artificial[,7]
env <- cbind(X,Y)

# run CCA function and save R^2 in R2

cca.XY <- CCA(bio,env)
R2 <- cca.XY$AV/cca.XY$TV

# compute adjusted R^2 by permuting the environmental variables 1000 times
# repeating CCA function with third input parameter=1 to just get R^2 each time

R2perm <- rep(0,1000)
set.seed(1234567)
for(iperm in 1:1000) {
  env.perm <- env[sample(1:nrow(env)),]
  R2perm[iperm] <- CCA(bio,env.perm,1)$AV/cca.XY$TV
  }

# adjusted R2 is the mean of the 1000 R^2's 

R2adj <- 1 - (1/(1 - mean(R2perm))) * (1 - R2)

# fuzzy code the two environmental variables into n.fuzzy categories, using function fuzzy.tri
# then repeat the above steps using different objects with suffix .fuzzy

n.fuzzy <- 7 
X.fuzzy <- fuzzy.tri(X,n.fuzzy)
Y.fuzzy <- fuzzy.tri(Y,n.fuzzy)

env.fuzzy <- cbind(X.fuzzy,Y.fuzzy)

# run CCA function and save R^2 in R2.fuzzy

cca.fuzzy <- CCA(bio,env.fuzzy)
R2.fuzzy  <- cca.fuzzy$AV/cca.fuzzy$TV

R2perm.fuzzy <- rep(0,1000)
set.seed(1234567)
for(iperm in 1:1000) {
  env.fuzzy.perm <- env.fuzzy[sample(1:nrow(env.fuzzy)),]
  R2perm.fuzzy[iperm] <- CCA(bio,env.fuzzy.perm,1)$AV/cca.fuzzy$TV
  }

R2adj.fuzzy <- 1 - (1/(1 - mean(R2perm.fuzzy))) * (1 - R2.fuzzy)


# ---------------------------------------------------------------------------------------------
# Analysis of the BarentsFish data set, showing how to code latitude (lat) and longitude (long) 
# into a 3x3 lattice of fuzzy categories, as in Figure 2
# depth and temperature are coded into 5 fuzzy categories each

Barents <- read.csv("BarentsFish.csv")
fish    <- as.matrix(Barents[,6:35])
lat     <- Barents[,2]
long    <- Barents[,3]
depth   <- Barents[,4]
temp    <- Barents[,5]

lat.f   <- fuzzy.tri(lat,3)
long.f  <- fuzzy.tri(long,3)
depth.f <- fuzzy.tri(depth,4)
temp.f  <- fuzzy.tri(temp,4)
latlong.f <- as.numeric(lat.f[1,] %o% long.f[1,])
for(i in 2:length(lat)) latlong.f <- rbind(latlong.f, as.numeric(lat.f[i,] %o% long.f[i,]))
ENV <- cbind(latlong.f, depth.f, temp.f)
colnames(ENV) <- c("SW","W","NW","S","C","N","SE","E","NE","d1","d2","d3","d4","t1","t2","t3","t4")

# run CCA function and save R^2 in R2

cca.fish <- CCA(as.matrix(fish),ENV)
R2.fish <- cca.fish$AV/cca.fish$TV

# compute adjusted R^2 by permuting the environmental variables 1000 times
# repeating CCA function with third input parameter=1 to just get R^2 each time

R2perm.fish <- rep(0,1000)
set.seed(1234567)
for(iperm in 1:1000) {
  ENV.perm <- ENV[sample(1:nrow(ENV)),]
  R2perm.fish[iperm] <- CCA(as.matrix(fish),ENV.perm,1)$AV/cca.fish$TV
  }

# adjusted R2 is the mean of the 1000 R^2's 

R2adj.fish <- 1 - (1/(1 - mean(R2perm.fish))) * (1 - R2.fish)

# plot BarentsFish results

# principal coordinates of rows, contribution coords of columns

samples.pc <- cca.fish$ARPC
species.cc <- cca.fish$ACCC

# main panel of Figure 5 - contribution coordinates are expanded by factor 3 for legibility

par(mar=c(4.2,4,2,2), cex.axis=0.8)
plot(rbind(samples.pc, 3*species.cc), type="n", asp=1, xlab="dimension 1", ylab="dimension 2") 
abline(v=0, col="grey", lty=2)
abline(h=0, col="grey", lty=2)
segments(0,0,3*species.cc[,1], 3*species.cc[,2],col="gray") 
points(samples.pc, col="lightgray", pch=19, cex=0.8)
fish.cex <- rep(0.6,ncol(fish))
fish.cex[c(6,8,13,16,21)] <- 0.9
text(3*species.cc, col="black", labels=colnames(fish), cex=fish.cex, font=4)

# zoomed panel of Figure 5 showing just environmental categories

par(mar=c(3,3,2,2), cex.axis=0.7)
plot(rbind(samples.pc, 3*species.cc), type="n", asp=1, xlab="", ylab="", xlim=c(-1.3, 1.3), ylim=c(-1.3, 1.3)) 
abline(v=0, col="grey", lty=2)
abline(h=0, col="grey", lty=2)
# compute environmental categories positions as weighted averages of the samples
ENV.cords <- t(ENV) %*% samples.pc[,1:2] / apply(ENV,2,sum)
lines(ENV.cords[10:13,], col="gray", lty=2, lwd=2)
lines(ENV.cords[14:17,], col="gray", lty=2, lwd=2)
ENV.color <- rep("black", ncol(ENV))
ENV.color[1:9] <- "darkgray"
text(ENV.cords, labels=colnames(ENV), col=ENV.color, cex=0.9, font=4)

# map of part of the Barents Sea showing sample locations

library(maps)       # mapping tool
library(mapdata)    # world map data

map("worldHires", interior = FALSE,  fill=TRUE, col="darkgray", xlim = c(0, 60), ylim = c(66, 81), mar=c(1,1,1,1))
symbols(long, lat, circles=rep(1,length(long)), inches=0.05, fg="black", bg="white", add=T)


 


